
#include <map>
#include <set>
#include <algorithm>
using namespace std;

#include <stdlib.h>

#include <libsaria/callback.h>
#include <libsaria/format.h>
#include "library.h"

#define MAX_TRACKS 100

static map<string, set<ino_t> > substr_index;
static set<ino_t> results;
static string cur_filter;
static bool filtered;

static void add_to_index(ino_t &inode, string &key)
{
	map<string, set<ino_t> >::iterator it;

	it = substr_index.find(key);
	if (it != substr_index.end())
		it->second.insert(inode);
	else {
		set<ino_t> to_insert;
		to_insert.insert(inode);
		substr_index.insert(pair<string, set<ino_t> >(key, to_insert));
	}
}

static void index_word(ino_t &inode, string &word)
{
	string substr, key;

	for (unsigned int i = 0; i < word.size(); i++) {
		substr = word.substr(i, word.size());
		for (unsigned int j = 1; j < substr.size() + 1; j++) {
			key = substr.substr(0, j);
			add_to_index(inode, key);
		}
	}
}

static void index_tag(ino_t &inode, list<string> *word_list)
{
	list<string>::iterator it;
	for (it = word_list->begin(); it != word_list->end(); it++)
		index_word(inode, *it);
}

IndexTask::IndexTask()  {};
IndexTask::~IndexTask() {};

void IndexTask::index_track(TrackTag *tag)
{
	track_list.push_back(tag);
}

void IndexTask::run_task()
{
	list<TrackTag *>::iterator it;
	ino_t inode;

	for (it = track_list.begin(); it != track_list.end(); it++) {
		inode = (*it)->get_inode();
		index_tag(inode, (*it)->get_title_words());
		index_tag(inode, (*it)->get_artist_words());
		index_tag(inode, (*it)->get_album_words());
	}
	if (cur_filter != "")
		libsaria::library::filter(cur_filter);
}

static void reindex_path(list<TrackTag> *tag_list)
{
	list<TrackTag>::iterator it;
	unsigned int i = 0, j = 0;
	IndexTask *task = new IndexTask();

	for (it = tag_list->begin(); it != tag_list->end(); it++) {
		task->index_track(&(*it));
		i++;
		if (i == MAX_TRACKS) {
			task->queue();
			task = new IndexTask();
			i = 0;
			j++;
		}
	}
	task->queue();
}

/*
 * Thanks to: http://stackoverflow.com/questions/1773526/in-place-c-set-intersection
 * for the set intersection algorithm!
 */
static void inplace_intersect(set<ino_t> *inodes)
{
	set<ino_t>::iterator it1 = results.begin();
	set<ino_t>::iterator it2 = inodes->begin();

	while ( (it1 != results.end()) && (it2 != inodes->end()) ) {
		if (*it1 < *it2)
			results.erase(it1++);
		else if (*it2 < *it1)
			it2++;
		else { /* *it1 == *it2 */
			it1++;
			it2++;
		}
	}

	/* Remove everything in results that wasn't in inodes */
	results.erase(it1, results.end());

}

static void do_filter(list<string> &terms)
{
	list<string>::iterator it;
	map<string, set<ino_t> >::iterator index_iter;

	for (it = terms.begin(); it != terms.end(); it++) {
		index_iter = substr_index.find(*it);
		/*
		 * Key not found means we don't need to filter anymore
		 * just clear the results set and return
		 */
		if (index_iter == substr_index.end()) {
			results.clear();
			return;
		}
		/*
		 * This is the first result, so the result set is empty.
		 * Taking an intersection will always give us an empty set
		 */
		if (it == terms.begin())
			results = index_iter->second;
		else
			inplace_intersect(&index_iter->second);
	}


}

namespace libsaria
{

	void library::reindex()
	{
		list<LibraryPath> *path_list = get_path_list();
		list<LibraryPath>::iterator it;

		for (it = path_list->begin(); it != path_list->end(); it++)
			reindex_path(it->get_list());
	}

	void library::filter(string &text)
	{
		list<string> key_list;

		format_text(text, key_list);
		results.clear();

		if (key_list.size() == 0)
			filtered = false;
		else {
			do_filter(key_list);
			filtered = true;
		}
		cur_filter = text;
		trigger_callback(REFILTER);
	}

	bool library::is_filtered()
	{
		return filtered;
	}

	bool library::is_visible(ino_t &inode)
	{
		set<ino_t>::iterator it;

		if (filtered == false)
			return true;
		it = results.find(inode);
		return it != results.end();
	}

	unsigned int library::filter_size()
	{
		return results.size();
	}

	ino_t library::index_rand()
	{
		set<ino_t>::iterator it = results.begin();
		unsigned int index = rand() % filter_size();

		for (unsigned int i = 0; i < index; i++)
			it++;
		return *it;
	}

}; /* Namespace: libsaria */
